@@ -42,7 +42,7 @@ class DiscreteFlowModel(object):
                 measurement_grid = obs[2] * 1.
             if weight_decay is not None:
                 # draw X% samples randomly from obs, where X is weight_decay^age
-                proportion = np.pow(weight_decay, age)
+                proportion = np.power(weight_decay, age)
                 obs_shuffled = np.random.permutation(obs[1])
                 N = int(proportion * len(obs_shuffled))
                 measurement_grid *= proportion
@@ -137,11 +137,11 @@ class DiscreteFlowModel(object):
         vymeas = np.concatenate((vymeas, np.zeros_like(vxmeas[:,:,:1])), axis=-1)
         # ---- vx -----
         # tile over [xx, yy, observation] - computing influence of each observation at each point
-        alphax = gamma*np.exp(-delta2)
+        alphax = np.exp(-gamma * delta2)
         TINY = 0.000001
         vxstar = np.sum(alphax * vxmeas, axis=-1) / (np.sum(alphax, axis=-1) + TINY)
         # ---- vy ----
-        alphay = gamma*np.exp(-delta2)
+        alphay = np.exp(-gamma * delta2)
         vystar = np.sum(alphay * vymeas, axis=-1) / (np.sum(alphay, axis=-1) + TINY)
         # ---- turbulence ---
         alphaabs = alphax
@@ -160,7 +160,8 @@ class DiscreteFlowModel(object):
             )
             rhogrid += gaussian
         # if a place is observed 10 times, and we observed 10 pedestrians, its average density is 1
-        measurement_grid[measurement_grid == 0] = np.inf
+        if isinstance(measurement_grid, np.ndarray):
+            measurement_grid[measurement_grid == 0] = np.inf
         rhogrid /= measurement_grid
         return vxstar, vystar, rhogrid, turbulence
 
@@ -201,7 +202,7 @@ class FlowBasedPlanner(NavigationPlanner):
         self.version = version
         self.nudge_max_speed = 0.1
         # RVO controller params
-        if self.version in [2, 6, 8, 9]:
+        if self.version in [2, 6, 8, 9, 11]:
             self.controller.safety_distance = 0.01
             self.controller.neighbor_dist = 3.
             self.controller.max_neighbors = 10
@@ -232,7 +233,7 @@ class FlowBasedPlanner(NavigationPlanner):
 
     def compute_cmd_vel(self, crowd, robot_pose, goal, dt,
                         measured_surface=None, crowd_odom=None, measurement_time=None,
-                        show_plot=True, debug=False, debug_dict=None):
+                        show_plot=True, debug=False, debug_dict=None, preplanned_traj=None):
         # If there's no crowd, just do rvo to the goal
         if len(crowd) == 0:
             return self.controller.compute_cmd_vel(crowd, robot_pose, goal, dt,
@@ -257,6 +258,8 @@ class FlowBasedPlanner(NavigationPlanner):
             crowd_sample = crowd_odom
         elif self.version in [8, 9, 10]:
             crowd_sample, measurement_grid = self.flow_model.get_observation_sample()
+        elif self.version in [11]:
+            crowd_sample, measurement_grid = self.flow_model.get_observation_sample(weight_decay=0.97)
         else:
             raise NotImplementedError
         vxgrid, vygrid, rhogrid, turbgrid = self.flow_model.get_discrete_model(
@@ -290,6 +293,8 @@ class FlowBasedPlanner(NavigationPlanner):
                                                    robot_static_obstacles_map,
                                                    self.flow_model.mu,
                                                    static_nudge_vel)
+        if preplanned_traj is not None:
+            trajectory = preplanned_traj
         debug_dict['trajectory'] = trajectory
         debug_dict['vrx'] = info['vrx']
         debug_dict['vry'] = info['vry']
@@ -302,7 +307,7 @@ class FlowBasedPlanner(NavigationPlanner):
             t, tx, ty, tvx, tvy = trajectory[0]
             target_vel = (tvx, tvy)
             nudge_case = np.linalg.norm(target_vel) < self.nudge_max_speed
-        elif self.version in [6, 7, 8, 9, 10]:
+        elif self.version in [6, 7, 8, 9, 10, 11]:
             nudge_case = len(trajectory) == 0
             if not nudge_case:
                 t, tx, ty, tvx, tvy = trajectory[0]
@@ -312,7 +317,7 @@ class FlowBasedPlanner(NavigationPlanner):
         # Run controller to get cmd_vel from target_vel
         if nudge_case:
             print("Nudge")
-            if self.version in [4, 6, 8, 9, 10]: # no solution - revert to baseline
+            if self.version in [4, 6, 8, 9, 10, 11]: # no solution - revert to baseline
                 cmd_vel = self.controller.compute_cmd_vel([], robot_pose, goal, dt,
                                                           show_plot=show_plot, debug=debug)
             elif self.version in [2, 3]:
@@ -322,11 +327,19 @@ class FlowBasedPlanner(NavigationPlanner):
                 cmd_vel = target_vel
             else:
                 raise NotImplementedError
+            nudge_trajectory = []
+            nvx, nvy = ((goal[0]-robot_pose[0]), (goal[1]-robot_pose[1]))
+            nvx = nvx / np.sqrt(nvx**2 + nvy**2)
+            nvy = nvy / np.sqrt(nvx**2 + nvy**2)
+            nudge_trajectory.append([0, robot_pose[0], robot_pose[1], nvx, nvy])
+            nudge_trajectory.append([10, goal[0], goal[1], 0, 0])
+            nudge_trajectory = np.array(nudge_trajectory).reshape((-1, 5))
+            debug_dict['trajectory'] = nudge_trajectory
         else:
             if self.version in [5, 9]:
                 cmd_vel = self.controller.run_rvo_step([], robot_pose, target_vel, dt,
                                                        show_plot=show_plot, debug=debug)
-            elif self.version in [1, 2, 3, 4, 6, 7, 8, 10]:
+            elif self.version in [1, 2, 3, 4, 6, 7, 8, 10, 11]:
                 cmd_vel = self.controller.run_rvo_step(crowd_odom, robot_pose, target_vel, dt,
                                                        show_plot=show_plot, debug=debug)
             else:
